package net.mc_pandacraft.java.plugin.pandacraftutils.data_model;

public class ModoHistoryElement extends SQLElement {
	
	private String modoName;
	private ActionType actionType;
	private long time;
	private String playerName;
	private Long value = null;
	private String message;
	

	public ModoHistoryElement(String modo, ActionType type, String player, String message) {
		super("pandacraft_modo_history");
		setModoName(modo);
		setActionType(type);
		setPlayerName(player);
		setMessage(message);
		time = System.currentTimeMillis();
	}
	
	ModoHistoryElement(int id, String modo, ActionType type, String player, String message) {
		super("pandacraft_modo_history", id);
		setModoName(modo);
		setActionType(type);
		setPlayerName(player);
		setMessage(message);
		time = System.currentTimeMillis();
	}

	@Override
	protected String[] getValues() {
		return new String[] {
				modoName,
				actionType.name(),
				String.valueOf(time),
				playerName,
				(value == null)?null:value.toString(),
				message
		};
	}

	@Override
	protected String[] getFieldsName() {
		return new String[] {
				"modoName",
				"actionType",
				"time",
				"playerName",
				"duration",
				"message"
		};
	}
	
	
	
	public String getModoName() {
		return modoName;
	}

	public void setModoName(String modoName) {
		if (modoName == null) throw new IllegalArgumentException("le paramètre ne peut être null");
		this.modoName = modoName;
	}



	public ActionType getActionType() {
		return actionType;
	}

	public void setActionType(ActionType actionType) {
		if (actionType == null) throw new IllegalArgumentException("le paramètre ne peut être null");
		this.actionType = actionType;
	}


	/**
	 * Retourne la durée de la sanction appliquée (en secondes), ou la somme d'argent retirée du compte
	 * @return
	 */
	public long getValue() {
		return value;
	}

	/**
	 * Value correspond soit à la durée de la sanction appliquée (en secondes), soit à la valeur de l'amende appliquée
	 * @param value
	 */
	public void setValue(Long value) {
		if (value != null && value.longValue() < 0) value = null;
		this.value = value;
	}



	public String getPlayerName() {
		return playerName;
	}

	public void setPlayerName(String playerName) {
		if (playerName == null) throw new IllegalArgumentException("le paramètre ne peut être null");
		this.playerName = playerName;
	}


	public String getMessage() {
		return message;
	}

	public void setMessage(String message) {
		if (message == null) throw new IllegalArgumentException("le paramètre ne peut être null");
		this.message = message;
	}







	public long getTime() {
		return time;
	}

	public void setTime(long time) {
		this.time = time;
	}







	public enum ActionType{
		BAN, UNBAN, MUTE, UNMUTE, JAIL, UNJAIL, REPORT, KICK, MONEY
	}

}
